<?php
/*
Plugin Name: Post Meta Manager
Plugin URI: http://andrewnorcross.com/plugins/post-meta-manager/
Description: Manage post meta keys in bulk
Version: 1.0
Author: norcross
Author URI: http://andrewnorcross.com
License: GPL v2

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


  
*/

// Start up the engine 
class PostMetaManager
{
    /**
     * Static property to hold our singleton instance
     * @var PostMetaManager
     */
    static $instance = false;


    /**
     * This is our constructor, which is private to force the use of
     * getInstance() to make this a Singleton
     *
     * @return PostMetaManager
     */
    private function __construct() {
        add_action      ( 'admin_menu',             array(&$this, 'menu_settings'   ) );
        add_action      ( 'admin_enqueue_scripts',  array(&$this, 'scripts_styles'  ), 10);
        add_action      ( 'wp_ajax_key_change',     array(&$this, 'key_change'      ) );
        add_action      ( 'wp_ajax_key_delete',     array(&$this, 'key_delete'      ) );

    }

    /**
     * If an instance exists, this returns it.  If not, it creates one and
     * retuns it.
     *
     * @return PostMetaManager
     */
     
    public static function getInstance() {
        if ( !self::$instance )
            self::$instance = new self;
        return self::$instance;
    }

    /**
     * Get plugin version to load in script
     *
     * @return PostMetaManager
     */

    public function plugin_version() {
        if ( ! function_exists( 'get_plugins' ) )
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        $plugin_folder = get_plugins( '/' . plugin_basename( dirname( __FILE__ ) ) );
        $plugin_file = basename( ( __FILE__ ) );
        return $plugin_folder[$plugin_file]['Version'];
    }

    /**
     * Scripts and stylesheets
     *
     * @return PostMetaManager
     */

    public function scripts_styles($hook) {
        
        $version = $this->plugin_version();

        $current_screen = get_current_screen();

        if ( 'tools_page_pmm-settings' == $current_screen->base ) {

            wp_enqueue_style( 'pmm-admin', plugins_url('/lib/css/pmm.admin.css', __FILE__), array(), $version, 'all' );
            wp_enqueue_script( 'pmm-ajax', plugins_url('/lib/js/pmm.ajax.js', __FILE__) , array('jquery'), $version, true );
        }

    }

    /**
     * build out settings page and meta boxes
     *
     * @return PostMetaManager
     */

    public function menu_settings() {
        add_submenu_page('tools.php', 'Post Meta Manager', 'Post Meta Manager', 'manage_options', 'pmm-settings', array( $this, 'pmm_display' ));
    }


    /**
     * run AJAX key change processing
     *
     * @return PostMetaManager
     */

    public function key_change() {

        // get keys from POST
        $key_old  = (isset($_POST['key_old']) && $_POST['key_old'] !== '' ? $_POST['key_old'] : false);
        $key_new  = (isset($_POST['key_new']) && $_POST['key_new'] !== '' ? $_POST['key_new'] : false);

        // set up return array for ajax responses
        $ret = array();

        // missing keys? bail

        if($key_old === false && $key_new === false) {
            $ret['success'] = false;
            $ret['process'] = 'NO_KEYS';
            $ret['message'] = 'You must enter a value in this field';
            echo json_encode($ret);
            die();
        }

        if($key_old === false) {
            $ret['success'] = false;
            $ret['process'] = 'NO_KEY_OLD';
            $ret['message'] = 'You must enter a value in this field';
            echo json_encode($ret);
            die();
        }

        if($key_new === false) {
            $ret['success'] = false;
            $ret['process'] = 'NO_KEY_NEW';
            $ret['message'] = 'You must enter a value in this field';
            echo json_encode($ret);
            die();
        }

        global $wpdb;
                                
        // run SQL query
    
        $key_query = $wpdb->query (
            $wpdb->prepare("
                UPDATE $wpdb->postmeta
                SET meta_key = REPLACE (meta_key, '".$key_old."', '".$key_new."')
            ")
        );


        if( $key_query == 0 ) {
            $ret['success'] = false;
            $ret['error']   = 'KEY_MISSING';
            $ret['message'] = 'There are no keys matching this criteria. Please try again.';
            echo json_encode($ret);
            die();
        }
        
        if( $key_query > 0 ) {
            $ret['success'] = true;
            $ret['updated'] = $key_query;
            $ret['message'] = $key_query.' entries have been updated.';
            echo json_encode($ret);
            die();
        }
        
    }

    /**
     * run AJAX key deletion processing
     *
     * @return PostMetaManager
     */

    public function key_delete() {

        // get keys from POST
        $key_kill  = (isset($_POST['key_kill']) && $_POST['key_kill'] !== '' ? $_POST['key_kill'] : false);

        // set up return array for ajax responses
        $ret = array();

        // missing keys? bail

        if($key_kill === false) {
            $ret['success'] = false;
            $ret['process'] = 'NO_KEY_ENTERED';
            $ret['message'] = 'You must enter a value in this field';
            echo json_encode($ret);
            die();
        }

        // so we have some keys. LETS DO THIS
        $args = array(
            'numberposts'     => -1,
            'meta_key'        => $key_kill,
            );
                
        $keys_array = get_posts( $args );

        $key_count  = count($keys_array);
        $key_exist  = ($key_count) > 0 ? true : false;

        // check count and process accordingly
        if($key_exist === true) {
            
            foreach ($keys_array as $key_post) {
                delete_post_meta($key_post->ID, $key_kill);
            }
            
            $ret['success'] = true;
            $ret['message'] = $key_count.' entries have been deleted.';
            echo json_encode($ret);
            die();

        } else {
            
            $ret['success'] = false;
            $ret['process'] = 'NO_KEYS_FOUND';
            $ret['message'] = 'There were no posts with the <strong><em>'.$key_kill.'</em></strong> meta key found. Please try again.';
            echo json_encode($ret);
            die();      
        
        }



    }

    /**
     * Display main options page structure
     *
     * @return PostMetaManager
     */
     
    public function pmm_display() { ?>
    
        <div class="wrap">
        <div class="icon32" id="icon-pmm"><br></div>
        <h2 class="pmm-title"><?php _e('Post Meta Manager') ?></h2>
        <div class="pmm-inner">

            <div class="pm_form_content">
            <p>Use the fields below to change or delete post meta keys (custom fields) in bulk.</p>
            </div>
           
            <form>
            <div class="pm_form_options">
            <h3>Change Meta Keys</h3>

                <table class="form-table pmm-table"><tbody>
                <tr>
                    <th><label for="meta_key_old"><?php _e('Existing Meta Key') ?></label></th>
                    <td>
                        <input type="text" class="regular-text meta_key_field" value="" id="meta_key_old" name="meta_key_old">
                        <span id="key_old_text" class="description"><?php _e('Enter the current meta key name you want to change') ?></span>
                    </td>
                </tr>

                <tr>
                    <th><label for="meta_key_new"><?php _e('New Meta Key Name') ?></label></th>
                    <td>
                        <input type="text" class="regular-text meta_key_field" value="" id="meta_key_new" name="meta_key_new">
                        <span id="key_new_text" class="description"><?php _e('Enter the new meta key name you want to use') ?></span>
                    </td>
                </tr>

                </tbody></table>
    
                <p class="process">
                <input id="pmm-change" type="button" class="button-secondary" value="<?php _e('Process Change') ?>" />
                <img class="ajax-loading btn-clear" src="<?php echo plugins_url('/lib/img/wpspin_light.gif', __FILE__); ?>">
                </p>
            
            </div>

            <form>
            <div class="pm_form_options">
            <h3>Delete Meta Keys</h3>

                <table class="form-table pmm-table"><tbody>
                <tr>
                    <th><label for="meta_key_kill"><?php _e('Meta Key Name') ?></label></th>
                    <td>
                        <input type="text" class="regular-text meta_key_field" value="" id="meta_key_kill" name="meta_key_kill">
                        <span id="key_kill_text" class="description"><?php _e('Enter the meta key name you want to remove') ?></span>
                    </td>
                </tr>

                </tbody></table>
    
                <p class="process">
                <input id="pmm-remove" type="button" class="button-secondary" value="<?php _e('Delete Keys') ?>" />
                <img class="ajax-loading btn-clear" src="<?php echo plugins_url('/lib/img/wpspin_light.gif', __FILE__); ?>">
                </p>
            
            </div>

            </form>

            <div class="pm_form_content">
            <p class="warning"><strong>NOTE: this is not reversible! Run a backup of your database if you aren't comfortable doing this.</strong></p>
            </div>

        </div>
        </div>
    
    <?php }

/// end class
}


// Instantiate our class
$PostMetaManager = PostMetaManager::getInstance();
